﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Channels;
using SampleBookMoveV2WebServiceClient.ODMoveServiceReference1;


namespace SampleBookMoveV2WebServiceClient
{

	static class Common 
	{
		// Just for the convenience of multiple tests against the same pro
		public const string ProNumber = "77741860116";
	}

	class Program
    {
        ODMoveDelegateClient service = new ODMoveDelegateClient();

        static void Main(string[] args)
        {
            Program client = new Program();
            
            // This is for OD internal testing against something other than production
            // It allows the certificate to go through without verification
            Util.SetCertificatePolicy();
            
            //  Your credentials are required for basic authentication
            client.service.ClientCredentials.UserName.UserName = "??????";                 
            client.service.ClientCredentials.UserName.Password = "??????";
            
            //****************************************************************
            // Uncomment the sample service call you want to execute
            //****************************************************************
            //client.bookMove();
            //client.changeDestinationAddress();
            //client.changeMoveDate();
            //client.guaranteeMove();
            //client.modifyAccessorials();
            //client.modifyTrailerCount();
            //client.reconsign();
            client.quoteDetails();
            //client.cancelMove();
            
            #if DEBUG
            	System.Threading.Thread.Sleep(5000);
            	Console.Write("Press any key to continue . . . ");
            	Console.ReadKey(true);
            #endif
        }

        private void bookMove()
        {
            address billAddr = new address();
            address orgAddr = new address();
            address destAddr = new address();
            bookRequest request = new bookRequest();

            orgAddr.address1 = "123 Origin Test";
            orgAddr.address2 = "#1";
            orgAddr.city = "Thomasville";
            orgAddr.country = "USA";
            orgAddr.name = "Wes Miller";
            orgAddr.postalCode = "27360";
            orgAddr.state = "NC";
            destAddr.address1 = "987 Destination Test";
            destAddr.city = "Beverly Hills";
            destAddr.country = "USA";
            destAddr.name = "Sandi Street";
            destAddr.postalCode = "90210";
            destAddr.state = "CA";
            billAddr = orgAddr;
            request.billingAddress = billAddr;
            request.originAddress = orgAddr;
            request.destinationAddress = destAddr;
            request.billingPhone1 = "1234567890";
            request.billingPhone2 = "9987654321";
            request.referenceNumber = "27245211";
            //request.email = "email@email.com";      //email used to obtain quote
            request.email = "benjamin.johnson@odfl.com";      //email used to obtain quote
            request.paymentType = "MO";
            request.accCodes = "";
            request.changeDate = "0";
            request.orgDropTime = "0";

            bookResponse response = service.bookMove(request);
            if (response.success)
            {
                Console.WriteLine("Success:");
                Console.WriteLine("Pro #: " + response.proNumber);
                Console.WriteLine("Coupon Amount: " + response.couponAmount);
            }
            else
            {
                Console.WriteLine("Error:");
                foreach (String s in response.errorMessages)
                {
                    Console.WriteLine(s);
                }
            }       
        }

        private void changeDestinationAddress()
        {
            changeAddressRequest request = new changeAddressRequest();
            request.address1 = "Change Addr 1";
            request.address2 = "Change Addr 2";
            request.referenceNumber = "24150220";

            homeMovingResponse response = service.changeConsigneeAddress(request);

            if (response.success)
            {
                Console.WriteLine("Success! Destination address has been changed.");
            }
            else
            {
                Console.WriteLine("Error:");
                foreach (String s in response.errorMessages)
                {
                    Console.WriteLine(s);
                }
            }
        }

        private void changeMoveDate()
        {
            changeDateRequest request = new changeDateRequest();
            request.changeDate = "20130425";
            request.referenceNumber = "24150220";

            homeMovingResponse response = service.changeMoveDate(request);

            if (response.success)
            {
                Console.WriteLine("Success! Move date has been changed.");
            }
            else
            {
                Console.WriteLine("Error:");
                foreach (String s in response.errorMessages)
                {
                    Console.WriteLine(s);
                }
            }
        }

        private void cancelMove()
        {
            cancelMoveRequest request = new cancelMoveRequest();
            request.proNumber = Common.ProNumber;

            Console.Write("Attempting to cancel move... ");
            homeMovingResponse response = service.cancelMove(request);
			
			outputResponse(response, () => {});
        }
        
        private void modifyAccessorials()
        {
        	modifyAccessorialsRequest request = new modifyAccessorialsRequest();
            
            request.proNumber = Common.ProNumber;
            request.guaranteeTime = 10;
            request.addAccessorials = "HH5HSO";
            
            // Or, use the remove parameter for removal
            // request.removeAccessorials = "HH5HSO";
			Console.Write("Attempting to modify accessorials... ");
            homeMovingResponse response = service.modifyAccessorials(request);

            outputResponse(response, () => {});
        }     
        
        private void modifyTrailerCount()
        {
        	modifyTrailerCountRequest request = new modifyTrailerCountRequest();
            
        	request.proNumber = Common.ProNumber;
            request.feetRequiredOnNewTrailer = 10;
            // 'A' for add, or 'R' for remove
            request.actionType = "A";
            
			Console.Write("Attempting to modify trailer count... ");
            modifyTrailerCountResponse response = service.modifyTrailerCount(request);

            outputResponse(response, () => {
                           	Console.WriteLine("New pro number: " + response.proNumber);
                           }
            );
                           
        }            

        private void reconsign()
        {
        	reconsignRequest request = new reconsignRequest();
            
        	address destAddr = new address();
            
            destAddr.address1 = "987 Reconsign Street";
            destAddr.city = "Beverly Hills";
            destAddr.country = "USA";
            destAddr.name = "Reconsign Name";
            destAddr.postalCode = "90212";
            destAddr.state = "CA";
            
            request.proNumber = Common.ProNumber;
            request.address = destAddr;
            
			Console.Write("Attempting to reconsign... ");
            reconsignResponse response = service.reconsign(request);
            
            outputResponse(response, () => {
                           	Console.WriteLine("Reconsignment amount: " + response.reconsignmentAmount);
                           }
            );
                           
        }  
        
         private void quoteDetails()
        {
        	quoteRequest request = new quoteRequest();
        	
        	request.quoteNumber = "29717266";
            
        	
            
			Console.Write("Attempting to get quote details");
            quoteResponse response = service.quoteLookup(request);
            
            outputResponse(response, () => {
                           	Console.WriteLine("Success = "+response.success);
                           }
            );
                           
        }  
        
        
        private void guaranteeMove()
        {
        	guaranteeRequest request = new guaranteeRequest();
            
            request.proNumber = Common.ProNumber;
            request.spotDate = new DateTime(2013,4,18);
            request.pickupDate = new DateTime(2013,4,19);
            request.deliveryDate = new DateTime(2013,4,21);
            
            // These are only required for a second trailer
            // request.spotDate2 = new DateTime(2013,4,18);
            // request.pickupDate2 = new DateTime(2013,4,19);
            // request.deliveryDate2 = new DateTime(2013,4,21);
            
            Console.Write("Attempting to guarantee move... ");
            homeMovingResponse response = service.guaranteeMove(request);
            
            outputResponse(response, () => {} );
                           
        }  
        
        private void outputResponse(homeMovingResponse response, Action showResults) {

            if (response.success)
            {
            	Console.WriteLine("Success!");
            	showResults();
            }
            else
            {
                Console.WriteLine("Error:");
                foreach (String s in response.errorMessages)
                {
                    Console.WriteLine(s);
                }
            }        	
        }

    }
}
