package com.odfl.ws.odmove.test;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.Date;
import org.apache.log4j.Logger;
import com.odfl.util.ODConstants;
import com.odfl.util.ODJdbc;
import com.odfl.ws.odmove.quotedetails.QuoteRequest;
import com.odfl.ws.odmove.quotedetails.QuoteResponse;
import org.junit.Test;

public class QuoteTest {	

	private static final Logger LOG = Logger.getLogger(QuoteTest.class);
	
	


	static String strSelect = "";
	static PreparedStatement pstmt = null;
	static ResultSet rs = null;
	static Connection conn2 = null;
	
	@Test
	public void testQuoteDetails() {
		QuoteRequest request = new QuoteRequest();
		request.setQuoteNumber("29717265");
		QuoteResponse response = execute(request);
		System.out.println("feet = "+response.getLinearFeet());
		System.out.println("origin zip = "+response.getOriginZip());
		System.out.println("destination zip = "+response.getDestinationZip());
		System.out.println("move date = "+response.getMoveDate());
		System.out.println("freight charge = "+response.getFreightCharge());
		System.out.println("linear feet charge = "+response.getLinearFeetCharge());
		System.out.println("final linear feet = "+response.getFinalLinearFeet());
		System.out.println("accessorial charge = "+response.getAccessorialCharge());
		System.out.println("final freight charge = "+response.getFinalFreightCharge());
		System.out.println("date quote generated = "+response.getDateQuoteGenerated());	
		System.out.println("expiration of quote = "+response.getExpirationOfQuote());
		System.out.println("error message = "+response.getErrorMessages());
		System.out.println("expired flag = "+response.isExpiredFlag());
		System.out.println("success flag = "+response.isSuccess());
	}

	// Method to get the quote response values
	public static QuoteResponse execute(QuoteRequest request) {
		QuoteResponse response = new QuoteResponse();
		String holdProNumber = "";	
		
		strSelect =("SELECT RQH_QUOTE#, RQH_LINEFT, RQH_OZIP, RQH_DZIP, RQH_DATE, RQH_MOVEDT, RQH_TOTCHG, RQH_PERLNR, RQH_EXDATE, RQH_PRONUM, PHH_TOTREV, PHH_ACCCHG FROM DTALIB.RATTQHDR " +
				"LEFT JOIN DTALIB.RATTHSTH ON RQH_PRONUM = PHH_PRO# " +
				"WHERE RQH_QUOTE# = ? and RQH_TARIFF = ? ");
		try {				

		//	conn2 = ODJdbc.getDSConn(ODConstants.DS_PROD);
			conn2 = ODJdbc.getConnection(ODConstants.SYSTEM, ODConstants.USER, ODConstants.PASS);
			if (conn2 == null) {
				LOG.info("conn2 is null");
				response.setExpiredFlag(true);
				return response;
			}

			pstmt = conn2.prepareStatement(strSelect.toString());
			pstmt.setString(1, request.getQuoteNumber());
			pstmt.setString(2, "0688");
			rs = pstmt.executeQuery();


			int count = 0;
			while (rs.next()) {						
				
				response.setSuccess(true);
				response.setLinearFeet(rs.getBigDecimal("RQH_LINEFT"));
				response.setOriginZip(rs.getString("RQH_OZIP"));
				response.setDestinationZip(rs.getString("RQH_DZIP"));
//				response.setMoveDate(rs.getDate("RQH_MOVEDT"));					
				response.setLinearFeetCharge(rs.getBigDecimal("RQH_PERLNR"));
//				response.setExpirationOfQuote(rs.getBigDecimal("RQH_EXDATE"));					
				response.setFreightCharge(rs.getBigDecimal("RQH_TOTCHG"));
				response.setFinalFreightCharge(rs.getBigDecimal("PHH_TOTREV"));
//				response.setDateQuoteGenerated(rs.getBigDecimal("RQH_DATE"));
				response.setAccessorialCharge(rs.getBigDecimal("PHH_ACCCHG"));
				holdProNumber = rs.getBigDecimal("RQH_PRONUM").toString();				
				
				String convertedQuoteGeneratedDate = convertDate(rs.getBigDecimal("RQH_DATE").toString());
				response.setDateQuoteGenerated(convertedQuoteGeneratedDate);
				
				String convertedExpirationQuoteDate = convertDate(rs.getBigDecimal("RQH_EXDATE").toString());
				response.setExpirationOfQuote(convertedExpirationQuoteDate);
				
				String convertedMoveDate = convertMoveDate(rs.getDate("RQH_MOVEDT"));
				response.setMoveDate(convertedMoveDate);

				boolean expirationDateValid = compareDates(rs.getBigDecimal("RQH_EXDATE").toString());
				if (expirationDateValid){
					response.setExpiredFlag(true);
				} else{
					response.setExpiredFlag(false);
				}


				

				count++;

			}




			if (count == 0) {
				response.setExpiredFlag(true);
				response.setSuccess(false);
				System.out.println("");
				return response;
			}


		} catch (SQLException se) {
			LOG.info("QuoteUtils - " + se);
			se.printStackTrace(System.out);

		} finally {
			ODJdbc.closeResultSet(rs);
			ODJdbc.closeStatement(pstmt);
			ODJdbc.closeConnection(conn2);
		}  // end try-catch-finally

		BigDecimal totalFeet = calculateFinalLinearFeet(holdProNumber);
		response.setFinalLinearFeet(totalFeet);

		return response;

	}

	// see if quote is expired
	public static boolean compareDates(String testExpirationDate){

		SimpleDateFormat sdf4 = new SimpleDateFormat("yyyyMMdd"); 
		Date expireyDate = null;
		Date currentDate = null;


		try{ 			

			expireyDate = sdf4.parse(testExpirationDate);
			System.out.println("Expiration Date = "+expireyDate);
			currentDate = new Date();
			System.out.println("Current Date = "+currentDate);

		}catch(Exception e)  
		{  
			e.printStackTrace();  
		}  

		if (expireyDate.before(currentDate)){
			return true;
		} else {
			return false;
		}





	}

	// convert date to proper format
	public static String convertDate(String testDate){
		String newTestDate = null;
		SimpleDateFormat sdf4 = new SimpleDateFormat("yyyyMMdd");
		SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("MM/dd/yyyy");
	        
		try{ 			

			Date origTestDate = sdf4.parse(testDate);			
			newTestDate = DATE_FORMAT.format(origTestDate);

		}catch(Exception e)  
		{  
			e.printStackTrace();  
		}  
		
		
		
		return newTestDate;
		
		
	}
	
	// convert date to proper format
	public static String convertMoveDate(Date testDate){
		String newTestDate = null;		
		SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("MM/dd/yyyy");
	        
		try{ 
			
			newTestDate = DATE_FORMAT.format(testDate);

		}catch(Exception e)  
		{  
			e.printStackTrace();  
		}  
		
		
		
		return newTestDate;
		
		
	}


	
	
	public static BigDecimal calculateFinalLinearFeet(String testHoldProNumber){
		
	//	conn2 = ODJdbc.getDSConn(ODConstants.DS_PROD);
		conn2 = ODJdbc.getConnection(ODConstants.SYSTEM, ODConstants.USER, ODConstants.PASS);
		if (conn2 == null) {
			LOG.info("conn2 is null");			
		}
		
		// Calling the stored procedure 		
		CallableStatement cs = null;
		int index = 0;

		try {
			cs = conn2.prepareCall("{CALL PGMLIB.INISPEXPR0107(?,?,?,?,?)}"); 
			cs.setString(++index,testHoldProNumber);						
			cs.registerOutParameter(++index, Types.DECIMAL);
			cs.registerOutParameter(++index, Types.DECIMAL);
			cs.registerOutParameter(++index, Types.DECIMAL);
			cs.registerOutParameter(++index, Types.CHAR);			
			cs.execute();
			String errMessage = (cs.getString(index));
			BigDecimal totalFeet = (cs.getBigDecimal(--index));
			BigDecimal movPro = (cs.getBigDecimal(--index)) ; 
			BigDecimal mstPro = (cs.getBigDecimal(--index)) ;
			
//			response.setDelvDate(cs.getBigDecimal(--index).toPlainString());


			return totalFeet;

		  // catch errors if any from calling the stored procedure	
		} catch (Exception se) {
			LOG.info("CalculateFinalLinearFeet Error - " + se);
			se.printStackTrace(System.out);

			return null;
		} finally {
			ODJdbc.closeStatement(cs);
			ODJdbc.closeDSConn(conn2);
		}  // end try-catch-finally
	
		
		
		
		
		
	}



}
